#ifndef INCLUDED_MyClientUser_H
#define INCLUDED_MyClientUser_H

// ======================================================================

class MyClientUser : public ClientUser
{
public:

	enum FileAction
	{
		FA_add,
		FA_modify,
		FA_delete,
		FA_count
	};

	enum PerforceState
	{
		PS_unknown,
		PS_unchanged,
		PS_add,
		PS_edit,
		PS_delete,
		PS_count
	};

public:

	static bool install();
	static void remove();

public:

	MyClientUser();
	MyClientUser(FileAction fileAction, char const * fileName);

	virtual void Message(Error *err);
	virtual void OutputError(const char *errBuf);
	virtual void OutputInfo(char level, const char *data);
	virtual void OutputBinary(const char *data, int length);
	virtual void OutputText(const char *data, int length);
	virtual void OutputStat(StrDict *varList);
	virtual void Prompt(StrPtr const & msg, StrBuf & rsp, int noEcho, Error * e);

	virtual FileSys * File(FileSysType type);

	void UpdatePerforce();

private:

	void runPerforceCommand(char const * commandName, int argc, char const * const * argv);

	void actionIgnore();
	void actionAdd();
	void actionEdit();
	void actionDelete();
	void actionRevert();
	void actionRevertUnchanged();
	void actionRevertEdit();
	void actionRevertDelete();

private:

	MyClientUser(MyClientUser &);
	MyClientUser & operator =(MyClientUser &);

	typedef void (MyClientUser::*Action)();
	static const Action ms_actionTable[FA_count][PS_count];

private:

	FileAction m_fileAction;
	char const * m_fileName;
	PerforceState m_perforceState;
	bool m_retry;
	char const * m_password;
};

// ======================================================================

#endif
